#-------------- slide example --------------
package User;
use base qw(Class::Tangram);
our $fields = {
        string => {
            name => {
                required => 1,
            },
        },
	idbif => {
		  gcos => undef,
		  passwd => undef,
		  uid => undef,
		  fav_artist => undef,
		 },
        ref => {
            group => {
                class => Group,
            },
        },
        set => {
            sec_groups => {
                class => Group,
            },
        },
    };

package Group;
use base qw(Class::Tangram);
our $fields = {
        string => {
            name => undef,
        },
        idbif => {
            passwd => undef,
            gid => undef,
        },
    };
#-------------- slide example ends --------------

# NOTE - the Tangram::IDBIF module modifies the schema passed to it,
# so this needs to be called first.  Normally this method is
# automatically called the first time you create an object of that
# type via Class->new()
Class::Tangram::import_schema($_) foreach qw(Group User);

package main;
use Tangram; # 2.08;
use Tangram::Relational;
use Tangram::Set;
use Tangram::Scalar;
use Tangram::Ref;
use Tangram::Schema;
use Tangram::IDBIF;

my $schema = Tangram::Schema->new
	(
	 # the default dumper is Storable, but we want it to be
	 # readable
	 sql => { dumper => "Data::Dumper" },
	 classes => [ map { $_ => { fields => ${"${_}::fields"} } }
	               qw(User Group) ],
	  # Hmm, database doesn't like a column/table name of "group"
	  # :-)
	  normalize => sub { local($_)=shift;
			     print "$_\n";
			     s/group/ggroup/i;
			     return $_ } );

# set to -pPasswd if required
$mysqlroot = "";

# create and connect to a database - replace with non-mysql commands
# if you don't use mysql.
system("yes | mysqladmin -uroot $mysqlroot drop svtestdb");
system("mysqladmin -uroot $mysqlroot create svtestdb");
system("echo 'delete from user where User = \"svtestdb\"; "
	."grant all privileges on svtestdb.* to "
	."svtestdb\@localhost identified by \"svtestdb\";' | "
	."mysql -uroot $mysqlroot mysql");
my @dsn = ("dbi:mysql:database=svtestdb", "svtestdb", "svtestdb");

$Tangram::TRACE = \*STDOUT;
Tangram::Relational->deploy($schema, @dsn);

BEGIN { $INC{"User.pm"} = $0; $INC{"Group.pm"} = $0; }
#-------------- slide example --------------
use User;
use Group;

eval { User->new() }; # exception - no name

my $group = Group->new
    ( name => "users",
      passwd => "*" );

eval { $group->set_gid("one") }; # not int!

my $user = User->new
    ( name => "sv",
      passwd => crypt("passw0rd", "x2"),
      gcos => "Sam Vilain",
      uid => 1000 + int(rand(2000)),
      group => $group );

print $user->group->name;  # prints users
#-------------- slide example pauses --------------

$group->set_gid(1000);
$user->set_uid(1042);

my $storage = Tangram::Storage->connect($schema, @dsn);

$storage->insert($user);
my @groups;
$storage->insert(@groups = 
		 (Group->new(gid => 1001,
			    name => "games",
			    passwd => crypt("letmein", "le")),
		  Group->new(gid => 1002,
			    name => "staff",
			    passwd => crypt("staff", "st")),
		 ),
		);
my $alice;
$storage->insert($alice =
		 User->new(name => "ac",
			   uid => "1043",
			   gcos => "Alice Cooper",
			   group => $group,
			   passwd => crypt("chains", "ch")));

#-------------- slide example resumes --------------
$user->sec_groups_insert(@groups);

$user->set_fav_artist($alice);
#-------------- slide example ends --------------


$storage->update($user);

# now tables will be in same state as slide!

$storage->disconnect();

print "Database is now in state of slide!  Press enter to clear: ";
my $junk = <>;
Tangram::Relational->retreat($schema, @dsn);
